<?php
/**
 * Provide a admin settings area view for the plugin
 *
 * This file is used to markup the admin-facing aspects of the plugin.
 *
 * @link       https://cobweb-security.com
 * @since      4.1.9
 *
 * @package    Cwis
 * @subpackage Cwis/admin/partials
 */
$error_message = null;

// Get a list of GDPR requests
$cwis_gdpr_requests = $this->get_class_instance('Cwis_GDPR_Requests');
$all_requests = $cwis_gdpr_requests->all_requests();

// Update the general settings (Enable GDPR Features)
if (($nonce = $this->filter_input_post('cwis-gdpr-general-nonce')) && wp_verify_nonce($nonce, 'cwis-gdpr-general')) {
    $cwis_gdpr_enabled = $this->filter_input_post('cwis_gdpr_enabled') ? 1 : 0;
    update_option('CWIS_GDPR_ENABLED', $cwis_gdpr_enabled);
}
$cwis_gdpr_enabled = get_option('CWIS_GDPR_ENABLED');

// Process the submitted user email
$user_email = sanitize_email($this->filter_input_post('user_email'));
if (!empty($user_email)) {

    // Request management
    if (!isset($all_requests[$user_email])) {
        $user = get_user_by('email', $user_email);
        if (!$user instanceof WP_User) {
            $error_message = esc_html__('Could not find a user with that email address.', CWIS_PLUGIN_SLUG);
        } else {
            // Add a request for that user email
            if ($cwis_gdpr_requests->add_request($user_email)) {
                $all_requests = $cwis_gdpr_requests->all_requests();
            } else {
                $error_message = esc_html__('The email address entered is not valid.', CWIS_PLUGIN_SLUG);
            }
        }
    } else {
        $error_message = esc_html__('The user with that email already placed a request.', CWIS_PLUGIN_SLUG);
    }
}
?>

<!-- Custom CSS -->
<style>
    .update-nag,
    #zing-warning,
    #wpbody-content .notice,
    #wpbody-content .updated {
        display: none;
    }

    /* Heading */
    h2 small {
        color: #72777c;
    }

    /* Helpers */
    .inline-block {
        display: inline-block;
    }
    .button>.dashicons {
        vertical-align: text-top;
    }

    /* Panels */
    .panel {
        clear: both;
        margin-bottom: 24px;
    }
    .panel>.panel-heading {
        background-image: -webkit-linear-gradient(top,#fff 0,#f6f7f7 100%);
        background-image: linear-gradient(to bottom,#fff 0,#f6f7f7 100%);
        padding: 15px 22px;
    }
    .panel>.panel-heading>h2 {
        color: #72777c;
        margin: 0;
    }
    .panel>.panel-body {
        background: #f6f7f7;
        border: 1px solid #eee;
        padding: 15px 22px;
    }
</style>

<div class="wrap">
    <div id="icon-plugins" class="icon32"><br></div>
    <h2><?php esc_html_e('WebDefender GDPR', CWIS_PLUGIN_SLUG); ?>
        <small>&mdash; <?php esc_html_e('Tools for GDPR Regulation', CWIS_PLUGIN_SLUG); ?></small>
    </h2>
    <hr />

    <div class="wrap-content">
        <div class="panel">
            <p>After four years of preparation and debate the General Data Protection Regulation (GDPR)
                was finally approved by the European Union (EU) Parliament on 14 April 2016.
                Enforcement date: 25 May 2018 - at which time those organizations in non-compliance
                may face heavy fines.
            </p>

            <p>Companies that collect data on citizens in EU countries will need to comply with
                strict new rules around protecting customer data by May 25, 2018. The GDPR is expected
                to set a new standard for consumer rights regarding their data, but companies will be
                challenged as they put systems and processes in place to comply.</p>

            <p>Compliance will cause some concerns and new expectations of security teams. For example,
                the GDPR takes a wide view of what constitutes personal identification information. Companies
                will need the same level of protection for things like an individual’s IP address or cookie data
                as they do for name, address and Social Security number.
            </p>

            <p><strong style="text-transform: uppercase"><?php esc_html_e('Note:', CWIS_PLUGIN_SLUG); ?></strong>
                <strong><?php
                    esc_html_e('Installing this plugin does not guarantee a full compliment with the GDPR. Please contact a GDPR consultant or a law firm to assess the necessary measures.',
                               CWIS_PLUGIN_SLUG);
                    ?></strong>
            </p>
        </div>

        <div class="panel" id="cwis-gdpr-general-tab">
            <div class="panel-heading clearfix">
                <h2 class="panel-title"><?php esc_html_e('General Settings', CWIS_PLUGIN_SLUG); ?></h2>
            </div>
            <div class="panel-body">
                <p><?php
                    esc_html_e('An integral part of your EU General Data Protection Regulation (GDPR) compliance project is producing appropriate documentation, which includes a privacy notice.',
                               CWIS_PLUGIN_SLUG); /* In addition, WebDefender GDPR enables customers to access system-generated logs. */
                    ?>
                    <?php
                    esc_html_e('Data Subject Access Requests (DSAR) tools enables you to fulfill GDPR requests. These capabilities allow website owners to respond to requests to access, rectify, delete, and export personal data.',
                               CWIS_PLUGIN_SLUG); /* In addition, WebDefender GDPR enables customers to access system-generated logs. */
                    ?>
                </p>

                <form method="post" id="cwis-gdpr-general-form"
                      action="<?php echo esc_url(admin_url('admin.php')); ?>?page=cwis-gdpr">
                          <?php
                          wp_nonce_field('cwis-gdpr-general', 'cwis-gdpr-general-nonce');
                          ?>
                    <table class="form-table" data-id="enable">
                        <tr>
                            <th scope="row">
                                <label for="cwis_gdpr_enabled"><?php
                                    esc_html_e('Enable GDPR Features', CWIS_PLUGIN_SLUG);
                                    ?></label>
                            </th>
                            <td>
                                <label class="gdpr-switch">
                                    <input value="1" type="checkbox"
                                           name="cwis_gdpr_enabled"
                                           id="cwis_gdpr_enabled" <?php
                                           checked(esc_attr($cwis_gdpr_enabled), 1);
                                           ?> onchange="(typeof this.form.submit === 'function') ?
                                                           this.form.submit() : this.form.submit.click()">
                                    <span class="gdpr-slider round"></span>
                                </label>
                            </td>
                        </tr>
                    </table>
                    <noscript>
                    <?php submit_button(); ?>
                    </noscript>
                </form>
            </div>
        </div>

        <div class="panel" id="cwis-gdpr-access-tab">
            <div class="panel-heading clearfix">
                <h2 class="panel-title"><?php esc_html_e('Access Personal Data', CWIS_PLUGIN_SLUG); ?></h2>
            </div>
            <div class="panel-body">
                <p><?php
                    esc_html_e('Individuals have the right to access their personal data, this is commonly referred to as subject access.',
                               CWIS_PLUGIN_SLUG);
                    ?>
                    <?php
                    esc_html_e('Individuals can make a subject access request verbally or in writing (website administrator have one month to respond to a request and cannot charge a fee to deal with a request in most circumstances).',
                               CWIS_PLUGIN_SLUG);
                    ?>
                    <?php esc_html_e('Read more about', CWIS_PLUGIN_SLUG); ?>
                    <a href="https://ico.org.uk/for-organisations/guide-to-the-general-data-protection-regulation-gdpr/individual-rights/right-of-access/" target="_blank"><?php
                        esc_html_e('Right of access', CWIS_PLUGIN_SLUG);
                        ?></a>.
                </p>
                <form method="post" id="cwis-gdpr-access-form"
                      action="<?php echo esc_url(admin_url('admin.php')); ?>?page=cwis-gdpr#cwis-gdpr-access-tab">
                    <div class="inside">
                        <?php
                        wp_nonce_field('cwis-gdpr-access', 'cwis-gdpr-access-nonce');
                        ?>
                        <strong><label for="gdpr-request-email-lookup">Search by email: </label></strong>
                        <input type="email" name="user_email" class="regular-text"
                               placeholder="<?php esc_html__('Enter e-mail address', CWIS_PLUGIN_SLUG); ?>"
                               value="<?php echo esc_html($user_email); ?>" required>
                               <?php
                               submit_button(esc_html__('Search', CWIS_PLUGIN_SLUG), 'primary', '', false);
                               ?>
                        <span class="spinner"></span>
                        <br />
                        <br />
                    </div>
                </form>

                <?php if (!empty($error_message)): ?>
                    <div class="error">
                        <p><?php echo esc_html__($error_message, CWIS_PLUGIN_SLUG); ?></p>
                    </div>
                <?php endif; ?>

                <?php if (!empty($all_requests)): ?>
                    <table class="widefat gdpr-request-table">
                        <thead>
                            <tr>
                                <th class="row-title"><?php esc_html_e('Type', CWIS_PLUGIN_SLUG); ?></th>
                                <th class="row-title"><?php esc_html_e('Email', CWIS_PLUGIN_SLUG); ?></th>
                                <th class="row-title"><?php esc_html_e('Requested at', CWIS_PLUGIN_SLUG); ?></th>
                                <th class="row-title"><?php esc_html_e('Status', CWIS_PLUGIN_SLUG); ?></th>
                                <th class="row-title"><?php esc_html_e('Actions', CWIS_PLUGIN_SLUG); ?></th>
                                <th class="row-title"><?php esc_html_e('Review', CWIS_PLUGIN_SLUG); ?></th>
                            </tr>
                        </thead>
                        <?php
                        $index = 0;
                        foreach ($all_requests as $email => $request):
                            ?>
                            <tr>
                                <td><?php echo esc_html(strtoupper($request['type'])); ?></td>
                                <td><?php echo esc_html($email); ?></td>
                                <td><?php echo esc_html(date("Y-m-d H:i", $request['ts_c'])); ?></td>
                                <td><?php echo esc_html(strtoupper($request['status'])); ?></td>
                                <td>
                                    <form class="frm-process-user-deletion" action="<?php echo esc_url(admin_url('admin-post.php')); ?>" method="post">
                                        <?php
                                        wp_nonce_field('gdpr-request-nonce', 'gdpr_cancel_delete_nonce');
                                        ?>
                                        <input type="hidden" name="action" value="gdpr_cancel_request">
                                        <input type="hidden" name="type" value="delete">
                                        <input type="hidden" name="user_email" value="<?php echo esc_attr($email) ?>">
                                        <?php
                                        submit_button(esc_html__('Cancel Request', 'gdpr'), 'delete', '', false)
                                        ?>
                                    </form>
                                    <?php if ('delete' === $request['type']) : ?>
                                        <form class="frm-process-user-deletion" action="<?php echo esc_url(admin_url('admin-post.php')); ?>" method="post">
                                            <?php
                                            wp_nonce_field('gdpr-request-delete-user', 'gdpr_delete_user');
                                            ?>
                                            <input type="hidden" name="action" value="gdpr_delete_user">
                                            <input type="hidden" name="user_email" value="<?php echo esc_attr($email) ?>">
                                            <?php
                                            submit_button(esc_html__('Delete User', 'gdpr'), 'primary', '', false)
                                            ?>
                                        </form>
                                    <?php else : ?>
                                        <form class="inline-block" method="post" id="cwis-gdpr-export-form"
                                              action="<?php echo esc_url(admin_url('admin-post.php')); ?>">
                                                  <?php
                                                  wp_nonce_field('cwis-gdpr-export', 'cwis-gdpr-export-nonce');
                                                  ?>
                                            <input type="hidden" name="action" value="cwis_gdpr_export">
                                            <input type="hidden" name="user_email" value="<?php echo esc_attr($email) ?>">
                                            <?php
                                            submit_button('JSON', 'primary', 'cwis-gdpr-export-format', false);
                                            ?>
                                            <?php
                                            submit_button('PDF', 'primary', 'cwis-gdpr-export-format', false);
                                            ?>
                                            <?php
                                            submit_button('XML', 'primary', 'cwis-gdpr-export-format', false);
                                            ?>
                                        </form>
                                    <?php endif; ?>
                                </td>
                                <td><button class="button gdpr-review" data-index="<?php echo $index; ?>">
                                        <span class="dashicons dashicons-arrow-down-alt2"></span></button>
                                </td>
                            </tr>
                            <tr class="review" data-index="<?php echo $index; ?>">
                                <td colspan="6">
                                    <div class="hidden">
                                        <?php if (!$cwis_gdpr_requests->user_has_content($email)) : ?>
                                            <p class="text-center"><?php
                                                echo esc_html_e('No content to review', CWIS_PLUGIN_SLUG);
                                                ?></p>
                                        <?php else : ?>
                                            <table class="widefat">
                                                <thead>
                                                    <tr>
                                                        <th><?php esc_html_e('Content Type', CWIS_PLUGIN_SLUG); ?></th>
                                                        <th class="text-center"><?php _e('Count', CWIS_PLUGIN_SLUG); ?></th>
                                                        <th class="text-center"><?php _e('Review', CWIS_PLUGIN_SLUG); ?></th>
                                                        <th class="text-center"><?php _e('Action', CWIS_PLUGIN_SLUG); ?></th>
                                                    </tr>
                                                </thead>
                                                <tbody>
                                                    <?php
                                                    $post_types = get_post_types(array(
                                                        'public' => true), 'objects');
                                                    ?>
                                                    <?php foreach ($post_types as $pt): ?>
                                                        <?php
                                                        $uid = get_user_by('email', $email);
                                                        if ($uid && $uid instanceof WP_User) {
                                                            $uid = $uid->ID;
                                                        }
                                                        $count = count_user_posts($uid, $pt->name);
                                                        if ('0' === $count) {
                                                            continue;
                                                        }
                                                        ?>
                                                        <tr>
                                                            <td class="row-title"><?php echo esc_attr($pt->label) ?></td>
                                                            <td class="text-center"><?php echo esc_attr($count) ?></td>
                                                            <td class="text-center">
                                                                <a href="<?php echo admin_url('edit.php?post_type=' . $pt->name . '&author=' . $uid); ?>" target="_blank" class="button"><?php echo esc_html($pt->labels->view_items); ?></a>
                                                            </td>
                                                            <td class="text-center">
                                                                <form method="post" class="gdpr-reassign-content">
                                                                    <?php
                                                                    wp_nonce_field('gdpr-reassign-content-action',
                                                                                   'gdpr_reassign_content_nonce')
                                                                    ?>
                                                                    <input type="hidden" name="user_email" value="<?php echo esc_attr($email) ?>">
                                                                    <input type="hidden" name="post_type" value="<?php echo esc_attr($pt->name); ?>">
                                                                    <input type="hidden" name="post_count" value="<?php echo esc_attr($count); ?>">
                                                                    <select name="reassign_to" class="gdpr-reassign">
                                                                        <option value="0"></option>
                                                                        <?php
                                                                        $admins = get_users(array(
                                                                            'role' => 'administrator'));
                                                                        ?>
                                                                        <?php foreach ($admins as $admin): ?>
                                                                            <option value="<?php echo esc_attr($admin->ID) ?>"><?php echo esc_html($admin->display_name) ?></option>
                                                                        <?php endforeach; ?>
                                                                    </select>
                                                                    <?php
                                                                    submit_button(esc_html__('Reassign',
                                                                                             CWIS_PLUGIN_SLUG),
                                                                                             'primary', '', false,
                                                                                             array('disabled' => true));
                                                                    ?>
                                                                    <span class="spinner"></span>
                                                                    <p class="hidden"><strong><?php
                                                                            esc_html_e('Resolved', CWIS_PLUGIN_SLUG);
                                                                            ?></strong></p>
                                                                </form>
                                                                <span class="spinner"></span>
                                                            </td>
                                                        </tr>
                                                    <?php endforeach; ?>
                                                    <?php
                                                    $comment_count = get_comments(array(
                                                        'author_email'       => $email,
                                                        'include_unapproved' => true,
                                                        'count'              => true,
                                                    ));

                                                    if ($comment_count) {
                                                        ?>
                                                        <tr>
                                                            <td class="row-title"><?php esc_html_e('Comments', CWIS_PLUGIN_SLUG); ?></td>
                                                            <td class="text-center"><?php echo esc_html($comment_count); ?></td>
                                                            <td class="text-center"><a href="<?php echo admin_url('edit-comments.php?comment_status=all&s=' . urlencode($email)); ?>" target="_blank" class="button"><?php
                                                                    _e('View Comments', CWIS_PLUGIN_SLUG);
                                                                    ?></a></td>
                                                            <td class="text-center">
                                                                <form method="post" class="gdpr-anonymize-comments">
                                                                    <?php
                                                                    wp_nonce_field('gdpr-anonymize-comments-action',
                                                                                   'gdpr_anonymize_comments_nonce')
                                                                    ?>
                                                                    <input type="hidden" name="user_email" value="<?php echo esc_attr($email) ?>">
                                                                    <input type="hidden" name="comment_count" value="<?php echo esc_attr($comment_count) ?>">
                                                                    <?php
                                                                    submit_button(esc_html__('Anonymize',
                                                                                             CWIS_PLUGIN_SLUG),
                                                                                             'primary', '', false)
                                                                    ?>
                                                                    <span class="spinner"></span>
                                                                    <p class="hidden"><strong><?php
                                                                            esc_html_e('Resolved', CWIS_PLUGIN_SLUG);
                                                                            ?></strong></p>
                                                                </form>
                                                            </td>
                                                        </tr>
                                                        <?php
                                                    }
                                                    ?>
                                                </tbody>
                                                <tr>
                                                </tr>
                                            </table>
                                        <?php endif; ?>
                                    </div>
                                </td>
                            </tr>
                            <?php
                            $index++;
                        endforeach;
                        ?>
                    </table>
                <?php endif; ?>
            </div>
        </div>

        <?php if ($cwis_gdpr_enabled) : ?>
            <div class="panel" id="cwis-gdpr-privacy-tab">
                <div class="panel-heading clearfix">
                    <h2 class="panel-title"><?php esc_html_e('Privacy Center', CWIS_PLUGIN_SLUG); ?></h2>
                </div>
                <div class="panel-body">
                    <p>Being transparent and providing accessible information to individuals about how you
                        will use their personal data is a key element of the Data Protection Act 1998 (DPA) and
                        the EU General Data Protection Regulation (GDPR). The most common way to provide this
                        information is in a privacy notice.
                        <?php esc_html_e('Read more about', CWIS_PLUGIN_SLUG); ?>
                        <a href="https://ico.org.uk/for-organisations/guide-to-data-protection/privacy-notices-transparency-and-control/" target="_blank"><?php
                            esc_html_e('Privacy notices, transparency and control', CWIS_PLUGIN_SLUG);
                            ?></a>.
                    </p>

                    <p><?php
                        esc_html_e('This section handles the privacy bar and some of the privacy preferences window.',
                                   CWIS_PLUGIN_SLUG);
                        ?><br>
                        <strong><?php esc_html_e('Important:', CWIS_PLUGIN_SLUG); ?></strong> <?php
                        esc_html_e('If the privacy banner text is not filled out, the privacy banner will not show up. Even if you registered your cookies.',
                                   CWIS_PLUGIN_SLUG);
                        ?>
                    </p>
                    <form action="options.php" method="post" class="gdpr-settings-form">
                        <?php settings_fields('cwis-gdpr'); ?>
                        <table class="form-table" data-id="general">
                            <tbody>
                                <tr>
                                    <th scope="row">
                                        <label for="gdpr_privacy_policy_page"><?php
                                            esc_html_e('Privacy Policy Page', CWIS_PLUGIN_SLUG);
                                            ?></label>
                                    </th>
                                    <td>
                                        <?php
                                        $pages = get_pages();
                                        $privacy_policy_page = get_option('gdpr_privacy_policy_page', 0);
                                        ?>
                                        <select name="gdpr_privacy_policy_page" id="gdpr_privacy_policy_page">
                                            <option value=""><?php esc_html_e('Select an option', CWIS_PLUGIN_SLUG); ?></option>
                                            <?php foreach ($pages as $page): ?>
                                                <option value="<?php echo esc_attr($page->ID) ?>" <?php
                                                selected($privacy_policy_page, $page->ID);
                                                ?>><?php echo esc_html($page->post_title); ?>
                                                </option>
                                            <?php endforeach ?>
                                        </select>
                                    </td>
                                </tr>
                                <tr>
                                    <th scope="row">
                                        <label for="gdpr_email_limit"><?php
                                            esc_html_e('Email Sending Limit', CWIS_PLUGIN_SLUG);
                                            ?></label>
                                    </th>
                                    <td><?php $limit = get_option('gdpr_email_limit', 100); ?>
                                        <input type="number" name="gdpr_email_limit" id="gdpr_email_limit" value="<?php echo esc_attr($limit); ?>">
                                    </td>
                                </tr>
                                <tr>
                                    <th scope="row">
                                        <label for="gdpr_deletion_needs_review"><?php
                                            esc_html_e('User deletion', CWIS_PLUGIN_SLUG)
                                            ?></label>
                                    </th>
                                    <td>
                                        <?php
                                        $needs_review = get_option('gdpr_deletion_needs_review', true);
                                        ?>
                                        <input value="1" type="checkbox" 
                                               name="<?php echo esc_attr('gdpr_deletion_needs_review'); ?>"
                                               id="gdpr_deletion_needs_review" <?php
                                               checked($needs_review, true);
                                               ?>>
                                        <label for="gdpr_deletion_needs_review"><?php
                                            esc_html_e('Send all deletion requests to the review table.',
                                                       CWIS_PLUGIN_SLUG);
                                            ?></label>
                                    </td>
                                </tr>
                                <tr>
                                    <th scope="row">
                                        <label for="gdpr_cookie_banner_content"><?php
                                            esc_html_e('Privacy Banner Text', CWIS_PLUGIN_SLUG);
                                            ?></label>
                                    </th>
                                    <td>
                                        <?php
                                        $privacy_bar_content = get_option('gdpr_cookie_banner_content', '');
                                        ?>
                                        <textarea name="gdpr_cookie_banner_content"
                                                  id="gdpr_cookie_banner_content"
                                                  cols="80" rows="6"><?php echo esc_html($privacy_bar_content);
                                        ?></textarea>
                                        <p class="description">
                                            <strong><?php esc_html_e('Example:', CWIS_PLUGIN_SLUG); ?></strong> <?php
                                            esc_html_e('This website intends to use cookies to improve the site and your experience. By continuing to browse the site you are agreeing to accept our use of cookies.',
                                                       CWIS_PLUGIN_SLUG);
                                            ?>
                                        </p>
                                    </td>
                                </tr>
                                <tr>
                                    <th scope="row">
                                        <label for="gdpr_cookie_privacy_excerpt"><?php
                                            esc_html_e('Privacy Excerpt', CWIS_PLUGIN_SLUG);
                                            ?>
                                        </label>
                                    </th>
                                    <td>
                                        <?php
                                        $privacy_excerpt = get_option('gdpr_cookie_privacy_excerpt', '');
                                        ?>
                                        <textarea name="gdpr_cookie_privacy_excerpt"
                                                  id="gdpr_cookie_privacy_excerpt"
                                                  cols="80" rows="6"><?php echo esc_html($privacy_excerpt);
                                        ?></textarea>
                                        <p class="description">
                                            <strong><?php esc_html_e('Note:', CWIS_PLUGIN_SLUG); ?></strong> <?php
                                            esc_html_e('This will appear in the consent section of the privacy preference window.',
                                                       CWIS_PLUGIN_SLUG);
                                            ?>
                                        </p>
                                    </td>
                                </tr>
                                <tr>
                                    <th scope="row">
                                        <label for=""><?php
                                            esc_html_e('WooCommerce', CWIS_PLUGIN_SLUG);
                                            ?>
                                        </label>
                                    </th>
                                    <td>
                                        <p>
                                            <label for="gdpr_add_consent_checkboxes_registration">
                                                <?php
                                                $add_checkboxes_to_registration = get_option('gdpr_add_consent_checkboxes_registration',
                                                                                             false);
                                                ?>
                                                <input value="1" type="checkbox" 
                                                       name="gdpr_add_consent_checkboxes_registration"
                                                       id="gdpr_add_consent_checkboxes_registration" <?php
                                                       checked($add_checkboxes_to_registration, true);
                                                       ?>>
                                                       <?php
                                                       esc_html_e('Add consent checkboxes to the registration page',
                                                                  CWIS_PLUGIN_SLUG);
                                                       ?>
                                            </label>
                                        </p>
                                        <p>
                                            <label for="gdpr_add_consent_checkboxes_checkout">
                                                <?php
                                                $add_checkboxes_to_checkout = get_option('gdpr_add_consent_checkboxes_checkout',
                                                                                         false);
                                                ?>
                                                <input value="1" type="checkbox" 
                                                       name="gdpr_add_consent_checkboxes_checkout"
                                                       id="gdpr_add_consent_checkboxes_checkout" <?php
                                                       checked($add_checkboxes_to_checkout, true);
                                                       ?>>
                                                       <?php
                                                       esc_html_e('Add consent checkboxes to the checkout registration form',
                                                                  CWIS_PLUGIN_SLUG);
                                                       ?>
                                            </label>
                                        </p>
                                    </td>
                                </tr>
                                <tr>
                                    <th scope="row">
                                        <label for=""><?php
                                            esc_html_e('Cookie Settings', CWIS_PLUGIN_SLUG);
                                            ?>
                                        </label>
                                    </th>
                                    <td><?php
                                        include_once 'inc-cookie-settings.php';
                                        include_once 'tmpl-cookies.php';
                                        ?>
                                    </td>
                                </tr>
                                <tr>
                                    <th scope="row">
                                        <label for=""><?php
                                            esc_html_e('Consent Settings', CWIS_PLUGIN_SLUG);
                                            ?>
                                        </label>
                                    </th>
                                    <td><?php
                                        include_once 'inc-consent-settings.php';
                                        include_once 'tmpl-consents.php';
                                        ?>
                                    </td>
                                </tr>
                            </tbody>
                        </table>
                        <?php submit_button(); ?>
                    </form>
                </div>
            </div>
        <?php endif; ?>

        <?php /*
          <div class="panel" id="cwis-gdpr-breach-tab">
          <div class="panel-heading clearfix">
          <h2 class="panel-title"><?php esc_html_e('Personal Data Breach', CWIS_PLUGIN_SLUG); ?></h2>
          </div>
          <div class="panel-body">
          <p><?php
          esc_html_e('The GDPR introduces a duty on all organisations to report certain types of personal data breach to the relevant supervisory authority. You must do this within 72 hours of becoming aware of the breach, where feasible.',
          CWIS_PLUGIN_SLUG);
          ?>
          <?php
          esc_html_e('If the breach is likely to result in a high risk of adversely affecting individuals’ rights and freedoms, you must also inform those individuals without undue delay.',
          CWIS_PLUGIN_SLUG);
          ?>
          <a href="https://ico.org.uk/for-organisations/guide-to-the-general-data-protection-regulation-gdpr/personal-data-breaches/" target="_blank"><?php
          esc_html_e('read more', CWIS_PLUGIN_SLUG);
          ?></a>
          </p>
          <form class="gdpr-data-breach-form" method="post"
          action="<?php echo esc_url(admin_url('admin-post.php')); ?>">
          <?php wp_nonce_field('gdpr-data-breach', 'gdpr_data_breach_nonce'); ?>
          <input type="hidden" name="action" value="gdpr_data_breach">
          <table class="form-table">
          <tr>
          <th><?php esc_html_e('Email content', CWIS_PLUGIN_SLUG); ?></th>
          <td>
          <textarea name="gdpr-data-breach-email-content" class="large-text" rows="5"></textarea>
          <span class="description"><?php
          esc_html_e('The content that the end user will see before the below information.',
          CWIS_PLUGIN_SLUG);
          ?></span>
          </td>
          </tr>
          <tr>
          <th><?php esc_html_e('Nature of the personal data breach', CWIS_PLUGIN_SLUG); ?></th>
          <td>
          <textarea name="gdpr-data-breach-nature" class="large-text" rows="5" required></textarea>
          <span class="description"><?php
          esc_html_e('Describe the nature of the personal data breach including where possible, the categories and approximate number of data subjects concerned and the categories and approximate number of personal data records concerned.',
          CWIS_PLUGIN_SLUG);
          ?></span>
          </td>
          </tr>
          <tr>
          <th><?php
          esc_html_e('Name and contact details of the data protection officer', CWIS_PLUGIN_SLUG);
          ?></th>
          <td>
          <textarea name="gdpr-name-contact-details-protection-officer" class="large-text" rows="5" required></textarea>
          <span class="description"><?php
          esc_html_e('Communicate the name and contact details of the data protection officer or other contact point where more information can be obtained.',
          CWIS_PLUGIN_SLUG);
          ?></span>
          </td>
          </tr>
          <tr>
          <th><?php esc_html_e('Likely consequences of the personal data breach', CWIS_PLUGIN_SLUG); ?></th>
          <td>
          <textarea name="gdpr-likely-consequences" class="large-text" rows="5" required></textarea>
          </td>
          </tr>
          <tr>
          <th><?php esc_html_e('Measures taken or proposed to be taken', CWIS_PLUGIN_SLUG); ?></th>
          <td>
          <textarea name="gdpr-measures-taken" class="large-text" rows="5" required></textarea>
          <span class="description"><?php
          esc_html_e('Describe the measures taken or proposed to be taken by the controller to address the personal data breach, including, where appropriate, measures to mitigate its possible adverse effects.',
          CWIS_PLUGIN_SLUG);
          ?></span>
          </td>
          </tr>
          </table>
          <?php
          submit_button(esc_html__('Send confirmation email', CWIS_PLUGIN_SLUG), 'primary', '', false);
          ?>
          </form>
          </div>
          </div>
         */ ?>

        <?php
        /*
          if (!empty($user_email)) {
          if (!empty($user) && $user instanceof WP_User) {
          echo '<h3>User Profile</h3>';
          echo '<table class="widefat">
          <tr>
          <td class="row-title" width="180">Username</td>
          <td>' . esc_html($user->user_login) . '</td>
          </tr>
          <tr>
          <td class="row-title">First Name</td>
          <td>' . esc_html($user->first_name) . '</td>
          </tr>
          <tr>
          <td class="row-title">Last Name</td>
          <td>' . esc_html($user->last_name) . '</td>
          </tr>
          <tr>
          <td class="row-title">Email</td>
          <td>' . esc_html($user->user_email) . '</td>
          </tr>
          <tr>
          <td class="row-title">Nickname</td>
          <td>' . esc_html($user->nickname) . '</td>
          </tr>
          <tr>
          <td class="row-title">Biographical Info</td>
          <td>' . esc_html($user->description) . '</td>
          </tr>
          <tr>
          <td class="row-title">Website</td>
          <td>' . esc_url($user->user_url) . '</td>
          </tr>
          <tr>
          <td class="row-title">Registered</td>
          <td>' . esc_html($user->user_registered) . '</td>
          </tr>
          <tr>
          <td class="row-title">Roles</td>
          <td>' . esc_html(implode(', ', $user->roles)) . '</td>
          </tr>
          <tr>
          <td class="row-title">Profile Picture</td>
          <td><img src="' . get_avatar_url($user->ID, array('size' => 90)) . '" alt="' . esc_html($user->user_email) . '" /></td>
          </tr>
          </table>';
          } else {
          ?>
          <div class="error">
          <p>We could not find a user with that email.</p>
          </div>
          <?php
          }
          }

          if (!empty($user_consents)) {
          echo '<h3>' . esc_html__('Consent Given', CWIS_PLUGIN_SLUG) . '</h3>';
          echo '<table class="widefat">
          <thead>
          <tr>
          <th width="180">' . esc_html__('Consent ID', CWIS_PLUGIN_SLUG) . '</th>
          </tr>
          </thead>';
          foreach ($user_consents as $v) {
          echo '<tr>';
          echo '<td class="row-title">' . esc_html($v) . '</td>';
          echo '</tr>';
          }
          echo '</table>';
          }

          if (!empty($comments)) {
          echo '<h3>' . esc_html__('Comments', CWIS_PLUGIN_SLUG) . '</h3>';
          foreach ($comments as $v) {
          echo '<table class="widefat">
          <thead>
          <tr>
          <th class="row-title" width="180">' . esc_html__('Comment Field', CWIS_PLUGIN_SLUG) . '</th>
          <th class="row-title">' . esc_html__('Comment Data', CWIS_PLUGIN_SLUG) . '</th>
          </tr>
          </thead>
          <tr>
          <td class="row-title">comment_author</td>
          <td>' . esc_html($v->comment_author) . '</td>
          </tr>
          <tr>
          <td class="row-title">comment_author_email</td>
          <td>' . esc_html($v->comment_author_email) . '</td>
          </tr>
          <tr>
          <td class="row-title">comment_author_url</td>
          <td>' . esc_html($v->comment_author_url) . '</td>
          </tr>
          <tr>
          <td class="row-title">comment_author_IP</td>
          <td>' . esc_html($v->comment_author_IP) . '</td>
          </tr>
          <tr>
          <td class="row-title">comment_date</td>
          <td>' . esc_html($v->comment_date) . '</td>
          </tr>
          <tr>
          <td class="row-title">comment_agent</td>
          <td>' . esc_html($v->comment_agent) . '</td>
          </tr>
          <tr>
          <td class="row-title">comment_content</td>
          <td>' . esc_html($v->comment_content) . '</td>
          </tr>
          </table><br>';
          }
          }

          if (!empty($usermeta)) {
          echo '<h3>' . esc_html__('Metadata', CWIS_PLUGIN_SLUG) . '</h3>';
          echo '<table class="widefat">
          <thead>
          <tr>
          <th width="180">' . esc_html__('Name', CWIS_PLUGIN_SLUG) . '</th>
          <th>' . esc_html__('Value', CWIS_PLUGIN_SLUG) . '</th>
          </tr>
          </thead>';
          foreach ($usermeta as $k => $v) {
          echo '<tr>';
          echo '<td class="row-title">' . esc_html($k) . '</td>';
          echo '<td>';
          foreach ($v as $value) {
          if (is_serialized($value)) {

          echo '<pre>' . print_r(maybe_unserialize($value), true) . '</pre><br />';
          } else {
          echo print_r($value, true) . '<br />';
          }
          }
          echo '</td>';
          echo '</tr>';
          }
          echo '</table>';
          }
         */
        ?>

        <div class="clear"></div>
    </div>
</div>
