<?php

/**
 * The file that defines the settings class.
 *
 * @link       https://cobweb-security.com
 * @since      4.1.4
 *
 * @package    Cwsd
 * @subpackage Cwsd/includes/defender
 * @author     Cobweb Security <cwis@cobweb-security.com>
 */
class Cwis_Defender_Settings
{

    // Defender options
    private $cwsd_options = array();
    private $cwsd_defaults = array(
        'smartwaf'   => false,
        //defender sub-options
        'antibot'    => false,
        'antispam'   => false,
        'bruteforce' => false,
        'hidefunc'   => false,
        'ipfilter'   => array(
            'allow' => '',
            'block' => ''
        )
    );

    /**
     * Initializes settings.
     *
     * @since   4.1.4
     * @access  public
     */
    public function __construct()
    {
        $this->merge(json_decode(get_option('CWIS_DEFENDER'), true));
    }

    /**
     * Returns an array with Defender options.
     *
     * @since   4.1.4
     * @access  public
     * 
     * @return  array
     */
    public function all()
    {
        return $this->cwsd_options;
    }

    /**
     * Returns value of a specific Defender option.
     *
     * @since   4.1.4
     * @access  public
     * 
     * @param   string $option
     * @param   mixed $default
     * @return  mixed
     */
    public function get($option = '', $default = null)
    {
        return strlen($option) && isset($this->cwsd_options[$option]) ?
                $this->cwsd_options[$option] : $default;
    }

    /**
     * Merges an array of options with Defender options.
     *
     * @since   4.1.4
     * @access  public
     * 
     * @param   array $options
     * @return  void
     */
    public function merge($options = array())
    {
        if (is_array($options)) {
            foreach ($options as $key => $value) {
                if (isset($this->cwsd_defaults[$key])) {
                    $this->cwsd_options[$key] = $value;
                }
            }
        }
    }

    /**
     * Sets Defender options.
     *
     * @since   4.1.4
     * @access  public
     * 
     * @param   string $option
     * @param   mixed $value
     * @return  void
     */
    public function set($option = '', $value = null)
    {
        if (strlen($option)) {
            $this->cwsd_options[$option] = $value;
        }
    }

    /**
     * Updates Defender options.
     *
     * @since   4.1.4
     * @access  public
     * 
     * @param   array $options
     * @return  void
     */
    public function update($options = array())
    {
        $this->cwsd_options = array_merge($this->all(), (array) $options);
        update_option('CWIS_DEFENDER', json_encode($this->cwsd_options));

        // Include the guarder class and save IP-filter settings
        require_once dirname(__FILE__) . '/class-guarder.php';
        $cwis_defender_guarder = new Cwis_Defender_Guarder();
        $cwis_defender_guarder->save_ipfilter_entries($this->get('ipfilter'));

        return $this->cwsd_options;
    }

}
