<?php

/**
 * The cronjob-specific functionality of the plugin.
 *
 * @link       https://cobweb-security.com
 * @since      3.3.3
 *
 * @package    Cwis
 * @subpackage Cwis/includes/scanner
 * @author     Cobweb Security <cwis@cobweb-security.com>
 */
class Cwis_Scanner_Cron extends Cwis_Core_Foundation
{

    public function __construct($plugin_name, $version, $loader = null)
    {
        parent::__construct($plugin_name, $version, $loader);
    }

    /**
     * Creates a new interval named "minutely".
     *
     * @since   3.0.1
     */
    public function cwis_scanner_cron_schedule()
    {
        $schedules = array(
            'minutely' => array(
                'interval' => 60,
                'display'  => 'Every Minute'
            )
        );

        return $schedules;
    }

    /**
     * Creates the CRON JOB callback function.
     *
     * @since   3.0.1
     */
    public function cwis_scanner_cron_callback()
    {
        $this->debug('cwis_scanner_cron_callback()');

        // Cronjob mode (3 for schedule)
        define('CWIS_IS_CRON', 3);

        // Scanner core signature
        define('CWIS_SIGNATURE', 'ezscan7objrldgpfvmqkdthyuwi4x2x5');

        // Absolute path to the root directory
        define('CWIS_ROOT_PATH', dirname(dirname(dirname(__FILE__))));

        // Absolute path to the working directory
        define('CWIS_WORK_PATH', CWIS_ROOT_PATH . DIRECTORY_SEPARATOR . 'cwis-scan');

        // Get upload path
        $upload_dir = wp_upload_dir();
        $upload_path = $upload_dir['basedir'] . '/cwis';

        // Absolute path to the temporary files directory
        define('CWIS_TEMP_PATH', $upload_path);

        // Path to the starting point for the scan
        define('CWIS_SCAN_PATH', substr(ABSPATH, 0, -1));

        // Require scanner defines
        if (!require_once(CWIS_WORK_PATH . DIRECTORY_SEPARATOR . 'cwis-defines.php')) {
            header('HTTP/1.0 500 Internal Server Error');
            die('Cannot load defines file!');
        }

        // Include scanner "compat" functions
        if (!require_once(CWIS_INCLUDES_PATH . DIRECTORY_SEPARATOR . 'cwis-compat.php')) {
            header('HTTP/1.0 500 Internal Server Error');
            die('Cannot load compat file!');
        }

        // Ready to lunch
        // ---------------------------
        // Require helpers and scheduler
        require_file('cwis-helpers.php');
        require_file('cwis-scheduler.php');

        // Init scheduler methods
        $scheduler = new CwisScheduler();
        $parsed = $scheduler->runCronJobChecks($task_name = 'schedule');
        if ($parsed->errmsg) {
            $scheduler->displayErrorMessage($parsed->errmsg);
            exit;
        }
        unset($scheduler);

        // Require scanner app file
        require_file('cwis-app.php');

        // Check required classes
        if (!class_exists('CwisApp')) {
            header('HTTP/1.0 500 Internal Server Error');
            die('Cannot load required app files!');
        }

        // Start cron job
        $app = new CwisApp();
        $app->construct(0, CWIS_SCAN_PATH);
        $app->startCron();

        wp_die(); // exit with a proper response
    }

}
