<?php

/**
 * The admin-specific functionality of the plugin.
 * 
 * Defines the plugin name, version, and hooks.
 *
 * @link       https://cobweb-security.com
 * @since      4.0.2.5
 *
 * @package    Cwis
 * @subpackage Cwis/includes/updater
 * @author     Cobweb Security <cwis@cobweb-security.com>
 */
class Cwis_Updater_Admin extends Cwis_Core_Foundation
{

    public function __construct($plugin_name, $version, $loader = null)
    {
        parent::__construct($plugin_name, $version, $loader);
    }

    /**
     * Register the function that contains the menu-building code for the admin area.
     *
     * @since   3.3.3
     */
    public function enqueue_updater_settings_menu()
    {

        // Add a link to the "Plugin Updates" settings
        $func_settings = array($this, 'display_updater_settings_page');
        add_options_page('WebDefender Updater', 'WebDefender Updater', 'administrator', 'cwis-updater', $func_settings);

        // Add a link to the main menu
        add_submenu_page('cwis-scanner', 'Updater', 'Updater', 'manage_options', 'cwis-updater', $func_settings);
    }

    /**
     * Creates the HTML output for the updater settings page (screen) displayed.
     * Page link: wp-admin/options-general.php?page=cwis-updater
     *
     * @since   3.3.0
     */
    public function display_updater_settings_page()
    {
        if (!current_user_can('manage_options')) {
            $message = esc_html__('You do not have sufficient permissions to access this page.', CWIS_PLUGIN_SLUG);
            $this->debug($message);
            wp_die($message);
        }

        require_once CWIS_PLUGIN_DIR_PATH . 'admin/partials/updater/settings-page.php';
    }

}
